@@ -2,7 +2,7 @@
 psycopg copy support
 """
 
-# Copyright (C) 2020-2021 The Psycopg Team
+# Copyright (C) 2020 The Psycopg Team
 
 import re
 import queue
@@ -11,72 +11,107 @@ import asyncio
 import threading
 from abc import ABC, abstractmethod
 from types import TracebackType
-from typing import TYPE_CHECKING, AsyncIterator, Iterator, Generic, Union
-from typing import Any, Dict, List, Match, Optional, Sequence, Type, Tuple
+from typing import Any, AsyncIterator, Dict, Generic, Iterator, List, Match, IO
+from typing import Optional, Sequence, Tuple, Type, Union, TYPE_CHECKING
 
 from . import pq
+from . import adapt
 from . import errors as e
-from .pq import ExecStatus
-from .abc import ConnectionType, PQGen, Transformer
-from .adapt import PyFormat
-from ._compat import create_task
+from .abc import Buffer, ConnectionType, PQGen, Transformer
+from ._compat import create_task, Self
+from .pq.misc import connection_summary
 from ._cmodule import _psycopg
 from ._encodings import pgconn_encoding
 from .generators import copy_from, copy_to, copy_end
 
 if TYPE_CHECKING:
-    from .pq.abc import PGresult
     from .cursor import BaseCursor, Cursor
     from .cursor_async import AsyncCursor
     from .connection import Connection  # noqa: F401
     from .connection_async import AsyncConnection  # noqa: F401
 
+PY_TEXT = adapt.PyFormat.TEXT
+PY_BINARY = adapt.PyFormat.BINARY
+
 TEXT = pq.Format.TEXT
 BINARY = pq.Format.BINARY
 
+COPY_IN = pq.ExecStatus.COPY_IN
+COPY_OUT = pq.ExecStatus.COPY_OUT
+
+ACTIVE = pq.TransactionStatus.ACTIVE
+
+# Size of data to accumulate before sending it down the network. We fill a
+# buffer this size field by field, and when it passes the threshold size
+# we ship it, so it may end up being bigger than this.
+BUFFER_SIZE = 32 * 1024
+
+# Maximum data size we want to queue to send to the libpq copy. Sending a
+# buffer too big to be handled can cause an infinite loop in the libpq
+# (#255) so we want to split it in more digestable chunks.
+MAX_BUFFER_SIZE = 4 * BUFFER_SIZE
+# Note: making this buffer too large, e.g.
+# MAX_BUFFER_SIZE = 1024 * 1024
+# makes operations *way* slower! Probably triggering some quadraticity
+# in the libpq memory management and data sending.
+
+# Max size of the write queue of buffers. More than that copy will block
+# Each buffer should be around BUFFER_SIZE size.
+QUEUE_SIZE = 1024
+
 
 class BaseCopy(Generic[ConnectionType]):
     """
-    Base implementation for copy user interface
+    Base implementation for the copy user interface.
 
     Two subclasses expose real methods with the sync/async differences.
 
     The difference between the text and binary format is managed by two
     different `Formatter` subclasses.
 
-    While the interface doesn't dictate it, both subclasses are implemented
-    with a worker to perform I/O related work, consuming the data provided in
-    the correct format from a queue, while the main thread is concerned with
-    formatting the data in copy format and adding it to the queue.
+    Writing (the I/O part) is implemented in the subclasses by a `Writer` or
+    `AsyncWriter` instance. Normally writing implies sending copy data to a
+    database, but a different writer might be chosen, e.g. to stream data into
+    a file for later use.
     """
 
-    # Max size of the write queue of buffers. More than that copy will block
-    # Each buffer around Formatter.BUFFER_SIZE size
-    QUEUE_SIZE = 1024
-
     formatter: "Formatter"
 
-    def __init__(self, cursor: "BaseCursor[ConnectionType, Any]"):
+    def __init__(
+        self,
+        cursor: "BaseCursor[ConnectionType, Any]",
+        *,
+        binary: Optional[bool] = None,
+    ):
         self.cursor = cursor
         self.connection = cursor.connection
         self._pgconn = self.connection.pgconn
 
-        tx = cursor._tx
-        assert tx.pgresult, "The Transformer doesn't have a PGresult set"
-        self._pgresult: "PGresult" = tx.pgresult
-
-        if self._pgresult.binary_tuples == pq.Format.TEXT:
-            self.formatter = TextFormatter(
-                tx, encoding=pgconn_encoding(self._pgconn)
-            )
+        result = cursor.pgresult
+        if result:
+            self._direction = result.status
+            if self._direction != COPY_IN and self._direction != COPY_OUT:
+                raise e.ProgrammingError(
+                    "the cursor should have performed a COPY operation;"
+                    f" its status is {pq.ExecStatus(self._direction).name} instead"
+                )
         else:
+            self._direction = COPY_IN
+
+        if binary is None:
+            binary = bool(result and result.binary_tuples)
+
+        tx: Transformer = getattr(cursor, "_tx", None) or adapt.Transformer(cursor)
+        if binary:
             self.formatter = BinaryFormatter(tx)
+        else:
+            self.formatter = TextFormatter(tx, encoding=pgconn_encoding(self._pgconn))
 
         self._finished = False
 
     def __repr__(self) -> str:
         cls = f"{self.__class__.__module__}.{self.__class__.__qualname__}"
-        info = pq.misc.connection_summary(self._pgconn)
+        info = connection_summary(self._pgconn)
         return f"<{cls} {info} at 0x{id(self):x}>"
 
     def _enter(self) -> None:
@@ -95,7 +130,7 @@ class BaseCopy(Generic[ConnectionType]):
 
         - On :sql:`COPY TO`, `!set_types()` allows to specify what types the
           operation returns. If `!set_types()` is not used, the data will be
-          reurned as unparsed strings or bytes instead of Python objects.
+          returned as unparsed strings or bytes instead of Python objects.
 
         - On :sql:`COPY FROM`, `!set_types()` allows to choose what type the
           database expects. This is especially useful in binary copy, because
@@ -103,22 +138,16 @@ class BaseCopy(Generic[ConnectionType]):
 
         """
         registry = self.cursor.adapters.types
-        oids = [
-            t if isinstance(t, int) else registry.get_oid(t) for t in types
-        ]
+        oids = [t if isinstance(t, int) else registry.get_oid(t) for t in types]
 
-        if self._pgresult.status == ExecStatus.COPY_IN:
-            self.formatter.transformer.set_dumper_types(
-                oids, self.formatter.format
-            )
+        if self._direction == COPY_IN:
+            self.formatter.transformer.set_dumper_types(oids, self.formatter.format)
         else:
-            self.formatter.transformer.set_loader_types(
-                oids, self.formatter.format
-            )
+            self.formatter.transformer.set_loader_types(oids, self.formatter.format)
 
     # High level copy protocol generators (state change of the Copy object)
 
-    def _read_gen(self) -> PQGen[memoryview]:
+    def _read_gen(self) -> PQGen[Buffer]:
         if self._finished:
             return memoryview(b"")
 
@@ -128,6 +157,11 @@ class BaseCopy(Generic[ConnectionType]):
 
         # res is the final PGresult
         self._finished = True
+
+        # This result is a COMMAND_OK which has info about the number of rows
+        # returned, but not about the columns, which is instead an information
+        # that was received on the COPY_OUT result at the beginning of COPY.
+        # So, don't replace the results in the cursor, just update the rowcount.
         nrows = res.command_tuples
         self.cursor._rowcount = nrows if nrows is not None else -1
         return memoryview(b"")
@@ -146,34 +180,62 @@ class BaseCopy(Generic[ConnectionType]):
 
         return row
 
-    def _end_copy_gen(self, exc: Optional[BaseException]) -> PQGen[None]:
-        bmsg: Optional[bytes]
-        if exc:
-            msg = f"error from Python: {type(exc).__qualname__} - {exc}"
-            bmsg = msg.encode(pgconn_encoding(self._pgconn), "replace")
-        else:
-            bmsg = None
+    def _end_copy_out_gen(self, exc: Optional[BaseException]) -> PQGen[None]:
+        if not exc:
+            return
 
-        res = yield from copy_end(self._pgconn, bmsg)
+        if self._pgconn.transaction_status != ACTIVE:
+            # The server has already finished to send copy data. The connection
+            # is already in a good state.
+            return
 
-        nrows = res.command_tuples
-        self.cursor._rowcount = nrows if nrows is not None else -1
-        self._finished = True
+        # Throw a cancel to the server, then consume the rest of the copy data
+        # (which might or might not have been already transferred entirely to
+        # the client, so we won't necessary see the exception associated with
+        # canceling).
+        self.connection.cancel()
+        try:
+            while (yield from self._read_gen()):
+                pass
+        except e.QueryCanceled:
+            pass
 
 
 class Copy(BaseCopy["Connection[Any]"]):
-    """Manage a :sql:`COPY` operation."""
+    """Manage a :sql:`COPY` operation.
+
+    :param cursor: the cursor where the operation is performed.
+    :param binary: if `!True`, write binary format.
+    :param writer: the object to write to destination. If not specified, write
+        to the `!cursor` connection.
+
+    Choosing `!binary` is not necessary if the cursor has executed a
+    :sql:`COPY` operation, because the operation result describes the format
+    too. The parameter is useful when a `!Copy` object is created manually and
+    no operation is performed on the cursor, such as when using ``writer=``\\
+    `~psycopg.copy.FileWriter`.
+
+    """
 
     __module__ = "psycopg"
 
-    def __init__(self, cursor: "Cursor[Any]"):
-        super().__init__(cursor)
-        self._queue: queue.Queue[Optional[bytes]] = queue.Queue(
-            maxsize=self.QUEUE_SIZE
-        )
-        self._worker: Optional[threading.Thread] = None
+    writer: "Writer"
 
-    def __enter__(self) -> "Copy":
+    def __init__(
+        self,
+        cursor: "Cursor[Any]",
+        *,
+        binary: Optional[bool] = None,
+        writer: Optional["Writer"] = None,
+    ):
+        super().__init__(cursor, binary=binary)
+        if not writer:
+            writer = LibpqWriter(cursor)
+
+        self.writer = writer
+        self._write = writer.write
+
+    def __enter__(self) -> Self:
         self._enter()
         return self
 
@@ -187,7 +249,7 @@ class Copy(BaseCopy["Connection[Any]"]):
 
     # End user sync interface
 
-    def __iter__(self) -> Iterator[memoryview]:
+    def __iter__(self) -> Iterator[Buffer]:
         """Implement block-by-block iteration on :sql:`COPY TO`."""
         while True:
             data = self.read()
@@ -195,7 +257,7 @@ class Copy(BaseCopy["Connection[Any]"]):
                 break
             yield data
 
-    def read(self) -> memoryview:
+    def read(self) -> Buffer:
         """
         Read an unparsed row after a :sql:`COPY TO` operation.
 
@@ -227,20 +289,22 @@ class Copy(BaseCopy["Connection[Any]"]):
         """
         return self.connection.wait(self._read_row_gen())
 
-    def write(self, buffer: Union[str, bytes]) -> None:
+    def write(self, buffer: Union[Buffer, str]) -> None:
         """
         Write a block of data to a table after a :sql:`COPY FROM` operation.
 
-        If the :sql:`COPY` is in binary format *buffer* must be `!bytes`. In
+        If the :sql:`COPY` is in binary format `!buffer` must be `!bytes`. In
         text mode it can be either `!bytes` or `!str`.
         """
         data = self.formatter.write(buffer)
-        self._write(data)
+        if data:
+            self._write(data)
 
     def write_row(self, row: Sequence[Any]) -> None:
         """Write a record to a table after a :sql:`COPY FROM` operation."""
         data = self.formatter.write_row(row)
-        self._write(data)
+        if data:
+            self._write(data)
 
     def finish(self, exc: Optional[BaseException]) -> None:
         """Terminate the copy operation and free the resources allocated.
@@ -249,63 +313,187 @@ class Copy(BaseCopy["Connection[Any]"]):
         by exit. It is available if, despite what is documented, you end up
         using the `Copy` object outside a block.
         """
-        # no-op in COPY TO
-        if self._pgresult.status == ExecStatus.COPY_OUT:
-            return
+        if self._direction == COPY_IN:
+            data = self.formatter.end()
+            if data:
+                self._write(data)
+            self.writer.finish(exc)
+            self._finished = True
+        else:
+            self.connection.wait(self._end_copy_out_gen(exc))
 
-        self._write_end()
-        self.connection.wait(self._end_copy_gen(exc))
 
-    # Concurrent copy support
+class Writer(ABC):
+    """
+    A class to write copy data somewhere.
+    """
+
+    @abstractmethod
+    def write(self, data: Buffer) -> None:
+        """
+        Write some data to destination.
+        """
+        ...
+
+    def finish(self, exc: Optional[BaseException] = None) -> None:
+        """
+        Called when write operations are finished.
+
+        If operations finished with an error, it will be passed to ``exc``.
+        """
+        pass
+
+
+class LibpqWriter(Writer):
+    """
+    A `Writer` to write copy data to a Postgres database.
+    """
+
+    def __init__(self, cursor: "Cursor[Any]"):
+        self.cursor = cursor
+        self.connection = cursor.connection
+        self._pgconn = self.connection.pgconn
+
+    def write(self, data: Buffer) -> None:
+        if len(data) <= MAX_BUFFER_SIZE:
+            # Most used path: we don't need to split the buffer in smaller
+            # bits, so don't make a copy.
+            self.connection.wait(copy_to(self._pgconn, data))
+        else:
+            # Copy a buffer too large in chunks to avoid causing a memory
+            # error in the libpq, which may cause an infinite loop (#255).
+            for i in range(0, len(data), MAX_BUFFER_SIZE):
+                self.connection.wait(
+                    copy_to(self._pgconn, data[i : i + MAX_BUFFER_SIZE])
+                )
+
+    def finish(self, exc: Optional[BaseException] = None) -> None:
+        bmsg: Optional[bytes]
+        if exc:
+            msg = f"error from Python: {type(exc).__qualname__} - {exc}"
+            bmsg = msg.encode(pgconn_encoding(self._pgconn), "replace")
+        else:
+            bmsg = None
+
+        try:
+            res = self.connection.wait(copy_end(self._pgconn, bmsg))
+        # The QueryCanceled is expected if we sent an exception message to
+        # pgconn.put_copy_end(). The Python exception that generated that
+        # cancelling is more important, so don't clobber it.
+        except e.QueryCanceled:
+            if not bmsg:
+                raise
+        else:
+            self.cursor._results = [res]
+
+
+class QueuedLibpqWriter(LibpqWriter):
+    """
+    A writer using a buffer to queue data to write to a Postgres database.
+
+    `write()` returns immediately, so that the main thread can be CPU-bound
+    formatting messages, while a worker thread can be IO-bound waiting to write
+    on the connection.
+    """
+
+    def __init__(self, cursor: "Cursor[Any]"):
+        super().__init__(cursor)
+
+        self._queue: queue.Queue[Buffer] = queue.Queue(maxsize=QUEUE_SIZE)
+        self._worker: Optional[threading.Thread] = None
+        self._worker_error: Optional[BaseException] = None
 
     def worker(self) -> None:
         """Push data to the server when available from the copy queue.
 
-        Terminate reading when the queue receives a None.
+        Terminate reading when the queue receives a false-y value, or in case
+        of error.
 
         The function is designed to be run in a separate thread.
         """
-        while 1:
-            data = self._queue.get(block=True, timeout=24 * 60 * 60)
-            if not data:
-                break
-            self.connection.wait(copy_to(self._pgconn, data))
-
-    def _write(self, data: bytes) -> None:
-        if not data:
-            return
+        try:
+            while True:
+                data = self._queue.get(block=True, timeout=24 * 60 * 60)
+                if not data:
+                    break
+                self.connection.wait(copy_to(self._pgconn, data))
+        except BaseException as ex:
+            # Propagate the error to the main thread.
+            self._worker_error = ex
 
+    def write(self, data: Buffer) -> None:
         if not self._worker:
             # warning: reference loop, broken by _write_end
             self._worker = threading.Thread(target=self.worker)
             self._worker.daemon = True
             self._worker.start()
 
-        self._queue.put(data)
+        # If the worker thread raies an exception, re-raise it to the caller.
+        if self._worker_error:
+            raise self._worker_error
 
-    def _write_end(self) -> None:
-        data = self.formatter.end()
-        self._write(data)
-        self._queue.put(None)
+        if len(data) <= MAX_BUFFER_SIZE:
+            # Most used path: we don't need to split the buffer in smaller
+            # bits, so don't make a copy.
+            self._queue.put(data)
+        else:
+            # Copy a buffer too large in chunks to avoid causing a memory
+            # error in the libpq, which may cause an infinite loop (#255).
+            for i in range(0, len(data), MAX_BUFFER_SIZE):
+                self._queue.put(data[i : i + MAX_BUFFER_SIZE])
+
+    def finish(self, exc: Optional[BaseException] = None) -> None:
+        self._queue.put(b"")
 
         if self._worker:
             self._worker.join()
             self._worker = None  # break the loop
 
+        # Check if the worker thread raised any exception before terminating.
+        if self._worker_error:
+            raise self._worker_error
+
+        super().finish(exc)
+
+
+class FileWriter(Writer):
+    """
+    A `Writer` to write copy data to a file-like object.
+
+    :param file: the file where to write copy data. It must be open for writing
+        in binary mode.
+    """
+
+    def __init__(self, file: IO[bytes]):
+        self.file = file
+
+    def write(self, data: Buffer) -> None:
+        self.file.write(data)
+
 
 class AsyncCopy(BaseCopy["AsyncConnection[Any]"]):
     """Manage an asynchronous :sql:`COPY` operation."""
 
     __module__ = "psycopg"
 
-    def __init__(self, cursor: "AsyncCursor[Any]"):
-        super().__init__(cursor)
-        self._queue: asyncio.Queue[Optional[bytes]] = asyncio.Queue(
-            maxsize=self.QUEUE_SIZE
-        )
-        self._worker: Optional[asyncio.Future[None]] = None
+    writer: "AsyncWriter"
 
-    async def __aenter__(self) -> "AsyncCopy":
+    def __init__(
+        self,
+        cursor: "AsyncCursor[Any]",
+        *,
+        binary: Optional[bool] = None,
+        writer: Optional["AsyncWriter"] = None,
+    ):
+        super().__init__(cursor, binary=binary)
+
+        if not writer:
+            writer = AsyncLibpqWriter(cursor)
+
+        self.writer = writer
+        self._write = writer.write
+
+    async def __aenter__(self) -> Self:
         self._enter()
         return self
 
@@ -317,14 +505,14 @@ class AsyncCopy(BaseCopy["AsyncConnection[Any]"]):
     ) -> None:
         await self.finish(exc_val)
 
-    async def __aiter__(self) -> AsyncIterator[memoryview]:
+    async def __aiter__(self) -> AsyncIterator[Buffer]:
         while True:
             data = await self.read()
             if not data:
                 break
             yield data
 
-    async def read(self) -> memoryview:
+    async def read(self) -> Buffer:
         return await self.connection.wait(self._read_gen())
 
     async def rows(self) -> AsyncIterator[Tuple[Any, ...]]:
@@ -337,55 +525,134 @@ class AsyncCopy(BaseCopy["AsyncConnection[Any]"]):
     async def read_row(self) -> Optional[Tuple[Any, ...]]:
         return await self.connection.wait(self._read_row_gen())
 
-    async def write(self, buffer: Union[str, bytes]) -> None:
+    async def write(self, buffer: Union[Buffer, str]) -> None:
         data = self.formatter.write(buffer)
-        await self._write(data)
+        if data:
+            await self._write(data)
 
     async def write_row(self, row: Sequence[Any]) -> None:
         data = self.formatter.write_row(row)
-        await self._write(data)
+        if data:
+            await self._write(data)
 
     async def finish(self, exc: Optional[BaseException]) -> None:
-        # no-op in COPY TO
-        if self._pgresult.status == ExecStatus.COPY_OUT:
-            return
+        if self._direction == COPY_IN:
+            data = self.formatter.end()
+            if data:
+                await self._write(data)
+            await self.writer.finish(exc)
+            self._finished = True
+        else:
+            await self.connection.wait(self._end_copy_out_gen(exc))
 
-        await self._write_end()
-        await self.connection.wait(self._end_copy_gen(exc))
 
-    # Concurrent copy support
+class AsyncWriter(ABC):
+    """
+    A class to write copy data somewhere (for async connections).
+    """
+
+    @abstractmethod
+    async def write(self, data: Buffer) -> None:
+        ...
+
+    async def finish(self, exc: Optional[BaseException] = None) -> None:
+        pass
+
+
+class AsyncLibpqWriter(AsyncWriter):
+    """
+    An `AsyncWriter` to write copy data to a Postgres database.
+    """
+
+    def __init__(self, cursor: "AsyncCursor[Any]"):
+        self.cursor = cursor
+        self.connection = cursor.connection
+        self._pgconn = self.connection.pgconn
+
+    async def write(self, data: Buffer) -> None:
+        if len(data) <= MAX_BUFFER_SIZE:
+            # Most used path: we don't need to split the buffer in smaller
+            # bits, so don't make a copy.
+            await self.connection.wait(copy_to(self._pgconn, data))
+        else:
+            # Copy a buffer too large in chunks to avoid causing a memory
+            # error in the libpq, which may cause an infinite loop (#255).
+            for i in range(0, len(data), MAX_BUFFER_SIZE):
+                await self.connection.wait(
+                    copy_to(self._pgconn, data[i : i + MAX_BUFFER_SIZE])
+                )
+
+    async def finish(self, exc: Optional[BaseException] = None) -> None:
+        bmsg: Optional[bytes]
+        if exc:
+            msg = f"error from Python: {type(exc).__qualname__} - {exc}"
+            bmsg = msg.encode(pgconn_encoding(self._pgconn), "replace")
+        else:
+            bmsg = None
+
+        try:
+            res = await self.connection.wait(copy_end(self._pgconn, bmsg))
+        # The QueryCanceled is expected if we sent an exception message to
+        # pgconn.put_copy_end(). The Python exception that generated that
+        # cancelling is more important, so don't clobber it.
+        except e.QueryCanceled:
+            if not bmsg:
+                raise
+        else:
+            self.cursor._results = [res]
+
+
+class AsyncQueuedLibpqWriter(AsyncLibpqWriter):
+    """
+    An `AsyncWriter` using a buffer to queue data to write.
+
+    `write()` returns immediately, so that the main thread can be CPU-bound
+    formatting messages, while a worker thread can be IO-bound waiting to write
+    on the connection.
+    """
+
+    def __init__(self, cursor: "AsyncCursor[Any]"):
+        super().__init__(cursor)
+
+        self._queue: asyncio.Queue[Buffer] = asyncio.Queue(maxsize=QUEUE_SIZE)
+        self._worker: Optional[asyncio.Future[None]] = None
 
     async def worker(self) -> None:
         """Push data to the server when available from the copy queue.
 
-        Terminate reading when the queue receives a None.
+        Terminate reading when the queue receives a false-y value.
 
-        The function is designed to be run in a separate thread.
+        The function is designed to be run in a separate task.
         """
-        while 1:
+        while True:
             data = await self._queue.get()
             if not data:
                 break
             await self.connection.wait(copy_to(self._pgconn, data))
 
-    async def _write(self, data: bytes) -> None:
-        if not data:
-            return
-
+    async def write(self, data: Buffer) -> None:
         if not self._worker:
             self._worker = create_task(self.worker())
 
-        await self._queue.put(data)
+        if len(data) <= MAX_BUFFER_SIZE:
+            # Most used path: we don't need to split the buffer in smaller
+            # bits, so don't make a copy.
+            await self._queue.put(data)
+        else:
+            # Copy a buffer too large in chunks to avoid causing a memory
+            # error in the libpq, which may cause an infinite loop (#255).
+            for i in range(0, len(data), MAX_BUFFER_SIZE):
+                await self._queue.put(data[i : i + MAX_BUFFER_SIZE])
 
-    async def _write_end(self) -> None:
-        data = self.formatter.end()
-        await self._write(data)
-        await self._queue.put(None)
+    async def finish(self, exc: Optional[BaseException] = None) -> None:
+        await self._queue.put(b"")
 
         if self._worker:
             await asyncio.gather(self._worker)
             self._worker = None  # break reference loops if any
 
+        await super().finish(exc)
+
 
 class Formatter(ABC):
     """
@@ -394,86 +661,80 @@ class Formatter(ABC):
 
     format: pq.Format
 
-    # Size of data to accumulate before sending it down the network
-    BUFFER_SIZE = 32 * 1024
-
     def __init__(self, transformer: Transformer):
         self.transformer = transformer
         self._write_buffer = bytearray()
         self._row_mode = False  # true if the user is using write_row()
 
     @abstractmethod
-    def parse_row(self, data: bytes) -> Optional[Tuple[Any, ...]]:
+    def parse_row(self, data: Buffer) -> Optional[Tuple[Any, ...]]:
         ...
 
     @abstractmethod
-    def write(self, buffer: Union[str, bytes]) -> bytes:
+    def write(self, buffer: Union[Buffer, str]) -> Buffer:
         ...
 
     @abstractmethod
-    def write_row(self, row: Sequence[Any]) -> bytes:
+    def write_row(self, row: Sequence[Any]) -> Buffer:
         ...
 
     @abstractmethod
-    def end(self) -> bytes:
+    def end(self) -> Buffer:
         ...
 
 
 class TextFormatter(Formatter):
-
-    format = pq.Format.TEXT
+    format = TEXT
 
     def __init__(self, transformer: Transformer, encoding: str = "utf-8"):
         super().__init__(transformer)
         self._encoding = encoding
 
-    def parse_row(self, data: bytes) -> Optional[Tuple[Any, ...]]:
+    def parse_row(self, data: Buffer) -> Optional[Tuple[Any, ...]]:
         if data:
             return parse_row_text(data, self.transformer)
         else:
             return None
 
-    def write(self, buffer: Union[str, bytes]) -> bytes:
+    def write(self, buffer: Union[Buffer, str]) -> Buffer:
         data = self._ensure_bytes(buffer)
         self._signature_sent = True
         return data
 
-    def write_row(self, row: Sequence[Any]) -> bytes:
+    def write_row(self, row: Sequence[Any]) -> Buffer:
         # Note down that we are writing in row mode: it means we will have
         # to take care of the end-of-copy marker too
         self._row_mode = True
 
         format_row_text(row, self.transformer, self._write_buffer)
-        if len(self._write_buffer) > self.BUFFER_SIZE:
+        if len(self._write_buffer) > BUFFER_SIZE:
             buffer, self._write_buffer = self._write_buffer, bytearray()
             return buffer
         else:
             return b""
 
-    def end(self) -> bytes:
+    def end(self) -> Buffer:
         buffer, self._write_buffer = self._write_buffer, bytearray()
         return buffer
 
-    def _ensure_bytes(self, data: Union[bytes, str]) -> bytes:
-        if isinstance(data, bytes):
-            return data
-
-        elif isinstance(data, str):
+    def _ensure_bytes(self, data: Union[Buffer, str]) -> Buffer:
+        if isinstance(data, str):
             return data.encode(self._encoding)
-
         else:
-            raise TypeError(f"can't write {type(data).__name__}")
+            # Assume, for simplicity, that the user is not passing stupid
+            # things to the write function. If that's the case, things
+            # will fail downstream.
+            return data
 
 
 class BinaryFormatter(Formatter):
-
-    format = pq.Format.BINARY
+    format = BINARY
 
     def __init__(self, transformer: Transformer):
         super().__init__(transformer)
         self._signature_sent = False
 
-    def parse_row(self, data: bytes) -> Optional[Tuple[Any, ...]]:
+    def parse_row(self, data: Buffer) -> Optional[Tuple[Any, ...]]:
         if not self._signature_sent:
             if data[: len(_binary_signature)] != _binary_signature:
                 raise e.DataError(
@@ -487,12 +748,12 @@ class BinaryFormatter(Formatter):
 
         return parse_row_binary(data, self.transformer)
 
-    def write(self, buffer: Union[str, bytes]) -> bytes:
+    def write(self, buffer: Union[Buffer, str]) -> Buffer:
         data = self._ensure_bytes(buffer)
         self._signature_sent = True
         return data
 
-    def write_row(self, row: Sequence[Any]) -> bytes:
+    def write_row(self, row: Sequence[Any]) -> Buffer:
         # Note down that we are writing in row mode: it means we will have
         # to take care of the end-of-copy marker too
         self._row_mode = True
@@ -502,13 +763,13 @@ class BinaryFormatter(Formatter):
             self._signature_sent = True
 
         format_row_binary(row, self.transformer, self._write_buffer)
-        if len(self._write_buffer) > self.BUFFER_SIZE:
+        if len(self._write_buffer) > BUFFER_SIZE:
             buffer, self._write_buffer = self._write_buffer, bytearray()
             return buffer
         else:
             return b""
 
-    def end(self) -> bytes:
+    def end(self) -> Buffer:
         # If we have sent no data we need to send the signature
         # and the trailer
         if not self._signature_sent:
@@ -527,17 +788,14 @@ class BinaryFormatter(Formatter):
         buffer, self._write_buffer = self._write_buffer, bytearray()
         return buffer
 
-    def _ensure_bytes(self, data: Union[bytes, str]) -> bytes:
-        if isinstance(data, bytes):
-            return data
-
-        elif isinstance(data, str):
-            raise TypeError(
-                "cannot copy str data in binary mode: use bytes instead"
-            )
-
+    def _ensure_bytes(self, data: Union[Buffer, str]) -> Buffer:
+        if isinstance(data, str):
+            raise TypeError("cannot copy str data in binary mode: use bytes instead")
         else:
-            raise TypeError(f"can't write {type(data).__name__}")
+            # Assume, for simplicity, that the user is not passing stupid
+            # things to the write function. If that's the case, things
+            # will fail downstream.
+            return data
 
 
 def _format_row_text(
@@ -553,11 +811,11 @@ def _format_row_text(
 
     for item in row:
         if item is not None:
-            dumper = tx.get_dumper(item, PyFormat.TEXT)
+            dumper = tx.get_dumper(item, PY_TEXT)
             b = dumper.dump(item)
             out += _dump_re.sub(_dump_sub, b)
         else:
-            out += br"\N"
+            out += rb"\N"
         out += b"\t"
 
     out[-1:] = b"\n"
@@ -572,7 +830,7 @@ def _format_row_binary(
         out = bytearray()
 
     out += _pack_int2(len(row))
-    adapted = tx.dump_sequence(row, [PyFormat.BINARY] * len(row))
+    adapted = tx.dump_sequence(row, [PY_BINARY] * len(row))
     for b in adapted:
         if b is not None:
             out += _pack_int4(len(b))
@@ -583,7 +841,7 @@ def _format_row_binary(
     return out
 
 
-def _parse_row_text(data: bytes, tx: Transformer) -> Tuple[Any, ...]:
+def _parse_row_text(data: Buffer, tx: Transformer) -> Tuple[Any, ...]:
     if not isinstance(data, bytes):
         data = bytes(data)
     fields = data.split(b"\t")
@@ -592,8 +850,8 @@ def _parse_row_text(data: bytes, tx: Transformer) -> Tuple[Any, ...]:
     return tx.load_sequence(row)
 
 
-def _parse_row_binary(data: bytes, tx: Transformer) -> Tuple[Any, ...]:
-    row: List[Optional[bytes]] = []
+def _parse_row_binary(data: Buffer, tx: Transformer) -> Tuple[Any, ...]:
+    row: List[Optional[Buffer]] = []
     nfields = _unpack_int2(data, 0)[0]
     pos = 2
     for i in range(nfields):
@@ -614,10 +872,9 @@ _unpack_int2 = struct.Struct("!h").unpack_from
 _unpack_int4 = struct.Struct("!i").unpack_from
 
 _binary_signature = (
-    # Signature, flags, extra length
-    b"PGCOPY\n\xff\r\n\0"
-    b"\x00\x00\x00\x00"
-    b"\x00\x00\x00\x00"
+    b"PGCOPY\n\xff\r\n\0"  # Signature
+    b"\x00\x00\x00\x00"  # flags
+    b"\x00\x00\x00\x00"  # extra length
 )
 _binary_trailer = b"\xff\xff"
 _binary_null = b"\xff\xff\xff\xff"
@@ -634,9 +891,7 @@ _dump_repl = {
 }
 
 
-def _dump_sub(
-    m: Match[bytes], __map: Dict[bytes, bytes] = _dump_repl
-) -> bytes:
+def _dump_sub(m: Match[bytes], __map: Dict[bytes, bytes] = _dump_repl) -> bytes:
     return __map[m.group(0)]
 
 
@@ -644,9 +899,7 @@ _load_re = re.compile(b"\\\\[btnvfr\\\\]")
 _load_repl = {v: k for k, v in _dump_repl.items()}
 
 
-def _load_sub(
-    m: Match[bytes], __map: Dict[bytes, bytes] = _load_repl
-) -> bytes:
+def _load_sub(m: Match[bytes], __map: Dict[bytes, bytes] = _load_repl) -> bytes:
     return __map[m.group(0)]
 
 
