@@ -47,7 +47,10 @@ def version() -> int:
 
 @impl.PQnoticeReceiver  # type: ignore
 def notice_receiver(arg: c_void_p, result_ptr: impl.PGresult_struct) -> None:
-    pgconn = cast(arg, POINTER(py_object)).contents.value()
+    pgconn = cast(arg, POINTER(py_object)).contents.value
+    if callable(pgconn):  # Not a weak reference on PyPy.
+        pgconn = pgconn()
+
     if not (pgconn and pgconn.notice_handler):
         return
 
@@ -262,7 +265,7 @@ class PGconn:
         self._ensure_pgconn()
         rv = impl.PQexec(self._pgconn_ptr, command)
         if not rv:
-            raise MemoryError("couldn't allocate PGresult")
+            raise e.OperationalError(f"executing query failed: {error_message(self)}")
         return PGresult(rv)
 
     def send_query(self, command: bytes) -> None:
@@ -286,7 +289,7 @@ class PGconn:
         self._ensure_pgconn()
         rv = impl.PQexecParams(*args)
         if not rv:
-            raise MemoryError("couldn't allocate PGresult")
+            raise e.OperationalError(f"executing query failed: {error_message(self)}")
         return PGresult(rv)
 
     def send_query_params(
@@ -427,7 +430,7 @@ class PGconn:
         self._ensure_pgconn()
         rv = impl.PQprepare(self._pgconn_ptr, name, command, nparams, atypes)
         if not rv:
-            raise MemoryError("couldn't allocate PGresult")
+            raise e.OperationalError(f"preparing query failed: {error_message(self)}")
         return PGresult(rv)
 
     def exec_prepared(
@@ -477,7 +480,9 @@ class PGconn:
             result_format,
         )
         if not rv:
-            raise MemoryError("couldn't allocate PGresult")
+            raise e.OperationalError(
+                f"executing prepared query failed: {error_message(self)}"
+            )
         return PGresult(rv)
 
     def describe_prepared(self, name: bytes) -> "PGresult":
@@ -486,7 +491,7 @@ class PGconn:
         self._ensure_pgconn()
         rv = impl.PQdescribePrepared(self._pgconn_ptr, name)
         if not rv:
-            raise MemoryError("couldn't allocate PGresult")
+            raise e.OperationalError(f"describe prepared failed: {error_message(self)}")
         return PGresult(rv)
 
     def send_describe_prepared(self, name: bytes) -> None:
@@ -504,7 +509,7 @@ class PGconn:
         self._ensure_pgconn()
         rv = impl.PQdescribePortal(self._pgconn_ptr, name)
         if not rv:
-            raise MemoryError("couldn't allocate PGresult")
+            raise e.OperationalError(f"describe portal failed: {error_message(self)}")
         return PGresult(rv)
 
     def send_describe_portal(self, name: bytes) -> None:
@@ -566,8 +571,9 @@ class PGconn:
         ptr = impl.PQnotifies(self._pgconn_ptr)
         if ptr:
             c = ptr.contents
-            return PGnotify(c.relname, c.be_pid, c.extra)
+            rv = PGnotify(c.relname, c.be_pid, c.extra)
             impl.PQfreemem(ptr)
+            return rv
         else:
             return None
 
