@@ -6,12 +6,16 @@ psycopg connection pool package
 
 from .pool import ConnectionPool
 from .pool_async import AsyncConnectionPool
+from .null_pool import NullConnectionPool
+from .null_pool_async import AsyncNullConnectionPool
 from .errors import PoolClosed, PoolTimeout, TooManyRequests
-from .version import __version__  # noqa: F401
+from .version import __version__ as __version__  # noqa: F401
 
 __all__ = [
     "AsyncConnectionPool",
+    "AsyncNullConnectionPool",
     "ConnectionPool",
+    "NullConnectionPool",
     "PoolClosed",
     "PoolTimeout",
     "TooManyRequests",
