@@ -16,7 +16,6 @@ from ._compat import Counter, Deque
 
 
 class BasePool(Generic[ConnectionType]):
-
     # Used to generate pool names
     _num_pool = 0
 
@@ -41,18 +40,18 @@ class BasePool(Generic[ConnectionType]):
         self,
         conninfo: str = "",
         *,
-        kwargs: Optional[Dict[str, Any]] = None,
-        min_size: int = 4,
-        max_size: Optional[int] = None,
-        open: bool = True,
-        name: Optional[str] = None,
-        timeout: float = 30.0,
-        max_waiting: int = 0,
-        max_lifetime: float = 60 * 60.0,
-        max_idle: float = 10 * 60.0,
-        reconnect_timeout: float = 5 * 60.0,
-        reconnect_failed: Optional[Callable[["BasePool[ConnectionType]"], None]] = None,
-        num_workers: int = 3,
+        kwargs: Optional[Dict[str, Any]],
+        min_size: int,
+        max_size: Optional[int],
+        open: bool,
+        name: Optional[str],
+        timeout: float,
+        max_waiting: int,
+        max_lifetime: float,
+        max_idle: float,
+        reconnect_timeout: float,
+        reconnect_failed: Optional[Callable[["BasePool[ConnectionType]"], None]],
+        num_workers: int,
     ):
         min_size, max_size = self._check_size(min_size, max_size)
 
@@ -65,8 +64,7 @@ class BasePool(Generic[ConnectionType]):
 
         self.conninfo = conninfo
         self.kwargs: Dict[str, Any] = kwargs or {}
-        self._reconnect_failed: Callable[["BasePool[ConnectionType]"], None]
-        self._reconnect_failed = reconnect_failed or (lambda pool: None)
+        self._reconnect_failed = reconnect_failed
         self.name = name
         self._min_size = min_size
         self._max_size = max_size
