@@ -21,7 +21,7 @@ from psycopg.pq import TransactionStatus
 from .base import ConnectionAttempt, BasePool
 from .sched import AsyncScheduler
 from .errors import PoolClosed, PoolTimeout, TooManyRequests
-from ._compat import Task, create_task, Deque
+from ._compat import Task, create_task, Deque, Self
 
 logger = logging.getLogger("psycopg.pool")
 
@@ -35,7 +35,19 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         connection_class: Type[AsyncConnection[Any]] = AsyncConnection,
         configure: Optional[Callable[[AsyncConnection[Any]], Awaitable[None]]] = None,
         reset: Optional[Callable[[AsyncConnection[Any]], Awaitable[None]]] = None,
-        **kwargs: Any,
+        kwargs: Optional[Dict[str, Any]] = None,
+        min_size: int = 4,
+        max_size: Optional[int] = None,
+        name: Optional[str] = None,
+        timeout: float = 30.0,
+        max_waiting: int = 0,
+        max_lifetime: float = 60 * 60.0,
+        max_idle: float = 10 * 60.0,
+        reconnect_timeout: float = 5 * 60.0,
+        reconnect_failed: Optional[
+            Callable[[BasePool[AsyncConnection[Any]]], None]
+        ] = None,
+        num_workers: int = 3,
     ):
         self.connection_class = connection_class
         self._configure = configure
@@ -54,7 +66,21 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         self._sched_runner: Optional[Task[None]] = None
         self._workers: List[Task[None]] = []
 
-        super().__init__(conninfo, **kwargs)
+        super().__init__(
+            conninfo,
+            kwargs=kwargs,
+            min_size=min_size,
+            max_size=max_size,
+            open=open,
+            name=name,
+            timeout=timeout,
+            max_waiting=max_waiting,
+            max_lifetime=max_lifetime,
+            max_idle=max_idle,
+            reconnect_timeout=reconnect_timeout,
+            reconnect_failed=reconnect_failed,
+            num_workers=num_workers,
+        )
 
         if open:
             self._open()
@@ -88,14 +114,14 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         self, timeout: Optional[float] = None
     ) -> AsyncIterator[AsyncConnection[Any]]:
         conn = await self.getconn(timeout=timeout)
-        t0 = monotonic()
         try:
+            t0 = monotonic()
             async with conn:
                 yield conn
         finally:
+            await self.putconn(conn)
             t1 = monotonic()
             self._stats[self._USAGE_MS] += int(1000.0 * (t1 - t0))
-            await self.putconn(conn)
 
     async def getconn(self, timeout: Optional[float] = None) -> AsyncConnection[Any]:
         logger.info("connection requested from %r", self.name)
@@ -204,8 +230,12 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         if not self._closed:
             return
 
-        # Throw a RuntimeError if the pool is open outside a running loop.
-        asyncio.get_running_loop()
+        try:
+            asyncio.get_running_loop()
+        except RuntimeError:
+            raise RuntimeError(
+                f"{type(self).__name__} open with no running loop"
+            ) from None
 
         self._check_open()
 
@@ -301,7 +331,7 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
                 timeout,
             )
 
-    async def __aenter__(self) -> "AsyncConnectionPool":
+    async def __aenter__(self) -> Self:
         await self.open()
         return self
 
@@ -344,6 +374,15 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
 
         while conns:
             conn = conns.pop()
+
+            # Check for expired connections
+            if conn._expire_at <= monotonic():
+                logger.info("discarding expired connection %s", conn)
+                await conn.close()
+                self.run_task(AddConnection(self))
+                continue
+
+            # Check for broken connections
             try:
                 await conn.execute("SELECT 1")
                 if conn.pgconn.transaction_status == TransactionStatus.INTRANS:
@@ -359,6 +398,9 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         """
         Called when reconnection failed for longer than `reconnect_timeout`.
         """
+        if not self._reconnect_failed:
+            return
+
         self._reconnect_failed(self)
 
     def run_task(self, task: "MaintenanceTask") -> None:
@@ -614,7 +656,7 @@ class AsyncClient:
 
     def __init__(self) -> None:
         self.conn: Optional[AsyncConnection[Any]] = None
-        self.error: Optional[Exception] = None
+        self.error: Optional[BaseException] = None
 
         # The AsyncClient behaves in a way similar to an Event, but we need
         # to notify reliably the flagger that the waiter has "accepted" the
@@ -634,8 +676,10 @@ class AsyncClient:
                     await asyncio.wait_for(self._cond.wait(), timeout)
                 except asyncio.TimeoutError:
                     self.error = PoolTimeout(
-                        f"couldn't get a connection after {timeout} sec"
+                        f"couldn't get a connection after {timeout:.2f} sec"
                     )
+                except BaseException as ex:
+                    self.error = ex
 
         if self.conn:
             return self.conn
