@@ -4,18 +4,20 @@ import ipaddress
 from math import isnan
 from uuid import UUID
 from random import choice, random, randrange
+from typing import Any, List, Set, Tuple, Union
 from decimal import Decimal
-from contextlib import contextmanager
-from collections import deque
+from contextlib import contextmanager, asynccontextmanager
 
 import pytest
 
 import psycopg
 from psycopg import sql
 from psycopg.adapt import PyFormat
-from psycopg._compat import asynccontextmanager
+from psycopg._compat import Deque
 from psycopg.types.range import Range
+from psycopg.types.json import Json, Jsonb
 from psycopg.types.numeric import Int4, Int8
+from psycopg.types.multirange import Multirange
 
 
 @pytest.fixture
@@ -36,7 +38,7 @@ class Faker:
 
     def __init__(self, connection):
         self.conn = connection
-        self._format = PyFormat.BINARY
+        self.format = PyFormat.BINARY
         self.records = []
 
         self._schema = None
@@ -45,18 +47,10 @@ class Faker:
         self._makers = {}
         self.table_name = sql.Identifier("fake_table")
 
-    @property
-    def format(self):
-        return self._format
-
-    @format.setter
-    def format(self, format):
-        self._format = format
-
     @property
     def schema(self):
         if not self._schema:
-            self._schema = self.choose_schema()
+            self.schema = self.choose_schema()
         return self._schema
 
     @schema.setter
@@ -71,9 +65,11 @@ class Faker:
     @property
     def types(self):
         if not self._types:
-            self._types = sorted(
-                self.get_supported_types(), key=lambda cls: cls.__name__
-            )
+
+            def key(cls: type) -> str:
+                return cls.__name__
+
+            self._types = sorted(self.get_supported_types(), key=key)
         return self._types
 
     @property
@@ -92,10 +88,7 @@ class Faker:
 
     @property
     def types_names(self):
-        types = [
-            t.as_string(self.conn).replace('"', "")
-            for t in self.types_names_sql
-        ]
+        types = [t.as_string(self.conn).replace('"', "") for t in self.types_names_sql]
         return types
 
     def _get_type_name(self, tx, schema, value):
@@ -118,21 +111,18 @@ class Faker:
 
     @property
     def create_stmt(self):
-        fields = []
+        field_values = []
         for name, type in zip(self.fields_names, self.types_names_sql):
-            fields.append(sql.SQL("{} {}").format(name, type))
+            field_values.append(sql.SQL("{} {}").format(name, type))
 
-        fields = sql.SQL(", ").join(fields)
-        return sql.SQL(
-            "create table {table} (id serial primary key, {fields})"
-        ).format(table=self.table_name, fields=fields)
+        fields = sql.SQL(", ").join(field_values)
+        return sql.SQL("create table {table} (id serial primary key, {fields})").format(
+            table=self.table_name, fields=fields
+        )
 
     @property
     def insert_stmt(self):
-        phs = [
-            sql.Placeholder(format=self.format)
-            for i in range(len(self.schema))
-        ]
+        phs = [sql.Placeholder(format=self.format) for i in range(len(self.schema))]
         return sql.SQL("insert into {} ({}) values ({})").format(
             self.table_name,
             sql.SQL(", ").join(self.fields_names),
@@ -142,17 +132,15 @@ class Faker:
     @property
     def select_stmt(self):
         fields = sql.SQL(", ").join(self.fields_names)
-        return sql.SQL("select {} from {} order by id").format(
-            fields, self.table_name
-        )
+        return sql.SQL("select {} from {} order by id").format(fields, self.table_name)
 
     @contextmanager
     def find_insert_problem(self, conn):
-        """Context manager to help finding a problematic vaule."""
+        """Context manager to help finding a problematic value."""
         try:
-            yield
+            with conn.transaction():
+                yield
         except psycopg.DatabaseError:
-            conn.rollback()
             cur = conn.cursor()
             # Repeat insert one field at time, until finding the wrong one
             cur.execute(self.drop_stmt)
@@ -166,8 +154,7 @@ class Faker:
                         if len(r) > 200:
                             r = f"{r[:200]}... ({len(r)} chars)"
                         raise Exception(
-                            f"value {r!r} at record {i} column0 {j}"
-                            f" failed insert: {e}"
+                            f"value {r!r} at record {i} column0 {j} failed insert: {e}"
                         ) from None
 
             # just in case, but hopefully we should have triggered the problem
@@ -176,9 +163,9 @@ class Faker:
     @asynccontextmanager
     async def find_insert_problem_async(self, aconn):
         try:
-            yield
+            async with aconn.transaction():
+                yield
         except psycopg.DatabaseError:
-            await aconn.rollback()
             acur = aconn.cursor()
             # Repeat insert one field at time, until finding the wrong one
             await acur.execute(self.drop_stmt)
@@ -192,8 +179,7 @@ class Faker:
                         if len(r) > 200:
                             r = f"{r[:200]}... ({len(r)} chars)"
                         raise Exception(
-                            f"value {r!r} at record {i} column0 {j}"
-                            f" failed insert: {e}"
+                            f"value {r!r} at record {i} column0 {j} failed insert: {e}"
                         ) from None
 
             # just in case, but hopefully we should have triggered the problem
@@ -206,11 +192,12 @@ class Faker:
         )
 
     def choose_schema(self, ncols=20):
-        schema = []
+        schema: List[Union[Tuple[type, ...], type]] = []
         while len(schema) < ncols:
             s = self.make_schema(choice(self.types))
             if s is not None:
                 schema.append(s)
+        self.schema = schema
         return schema
 
     def make_records(self, nrecords):
@@ -221,8 +208,7 @@ class Faker:
             return tuple(self.example(spec) for spec in self.schema)
         else:
             return tuple(
-                self.make(spec) if random() > nulls else None
-                for spec in self.schema
+                self.make(spec) if random() > nulls else None for spec in self.schema
             )
 
     def assert_record(self, got, want):
@@ -232,16 +218,13 @@ class Faker:
             m = self.get_matcher(spec)
             m(spec, g, w)
 
-    def get_supported_types(self):
+    def get_supported_types(self) -> Set[type]:
         dumpers = self.conn.adapters._dumpers[self.format]
         rv = set()
         for cls in dumpers.keys():
             if isinstance(cls, str):
                 cls = deep_import(cls)
-            if (
-                issubclass(cls, psycopg.types.multirange.Multirange)
-                and self.conn.info.server_version < 140000
-            ):
+            if issubclass(cls, Multirange) and self.conn.info.server_version < 140000:
                 continue
 
             rv.add(cls)
@@ -252,7 +235,7 @@ class Faker:
 
         return rv
 
-    def make_schema(self, cls):
+    def make_schema(self, cls: type) -> Union[Tuple[type, ...], type, None]:
         """Create a schema spec from a Python type.
 
         A schema specifies what Postgres type to generate when a Python type
@@ -281,9 +264,7 @@ class Faker:
             self._makers[cls] = meth
             return meth
         else:
-            raise NotImplementedError(
-                f"cannot make fake objects of class {cls}"
-            )
+            raise NotImplementedError(f"cannot make fake objects of class {cls}")
 
     def get_matcher(self, spec):
         cls = spec if isinstance(spec, type) else spec[0]
@@ -348,24 +329,22 @@ class Faker:
     def make_datetime(self, spec):
         # Add a day because with timezone we might go BC
         dtmin = dt.datetime.min + dt.timedelta(days=1)
-
-        # Comparisons might get unreliable too far in the future
-        # https://bugs.python.org/issue45347
-        # delta = dt.datetime.max - dtmin
-        delta = dt.timedelta(days=3000 * 365)
-
+        delta = dt.datetime.max - dtmin
         micros = randrange((delta.days + 1) * 24 * 60 * 60 * 1_000_000)
         rv = dtmin + dt.timedelta(microseconds=micros)
         if spec[1]:
             rv = rv.replace(tzinfo=self._make_tz(spec))
         return rv
 
+    def match_datetime(self, spec, got, want):
+        # Comparisons with different timezones is unreliable: certain pairs
+        # are reported different but their delta is 0
+        # https://bugs.python.org/issue45347
+        assert not (got - want)
+
     def make_Decimal(self, spec):
         if random() >= 0.99:
-            if self.conn.info.server_version >= 140000:
-                return Decimal(choice(["NaN", "sNaN", "Inf", "-Inf"]))
-            else:
-                return Decimal(choice(["NaN", "sNaN"]))
+            return Decimal(choice(self._decimal_special_values()))
 
         sign = choice("+-")
         num = choice(["0.zd", "d", "d.d"])
@@ -388,6 +367,27 @@ class Faker:
         else:
             assert got == want
 
+    def _decimal_special_values(self):
+        values = ["NaN", "sNaN"]
+
+        if self.conn.info.vendor == "PostgreSQL":
+            if self.conn.info.server_version >= 140000:
+                values.extend(["Inf", "-Inf"])
+        elif self.conn.info.vendor == "CockroachDB":
+            if self.conn.info.server_version >= 220100:
+                values.extend(["Inf", "-Inf"])
+        else:
+            pytest.fail(f"unexpected vendor: {self.conn.info.vendor}")
+
+        return values
+
+    def schema_Enum(self, cls):
+        # TODO: can't fake those as we would need to create temporary types
+        return None
+
+    def make_Enum(self, spec):
+        return None
+
     def make_float(self, spec, double=True):
         if random() <= 0.99:
             # These exponents should generate no inf
@@ -397,21 +397,26 @@ class Faker:
                 else f"{choice('-+')}0.{randrange(1 << 22)}e{randrange(-37,38)}"
             )
         else:
-            return choice(
-                (0.0, -0.0, float("-inf"), float("inf"), float("nan"))
-            )
+            return choice((0.0, -0.0, float("-inf"), float("inf"), float("nan")))
 
     def match_float(self, spec, got, want, approx=False, rel=None):
         if got is not None and isnan(got):
             assert isnan(want)
+        else:
+            if approx or self._server_rounds():
+                assert got == pytest.approx(want, rel=rel)
+            else:
+                assert got == want
+
+    def _server_rounds(self):
+        """Return True if the connected server perform float rounding"""
+        if self.conn.info.vendor == "CockroachDB":
+            return True
         else:
             # Versions older than 12 make some rounding. e.g. in Postgres 10.4
             # select '-1.409006204063909e+112'::float8
             #      -> -1.40900620406391e+112
-            if not approx and self.conn.info.server_version >= 120000:
-                assert got == want
-            else:
-                assert got == pytest.approx(want, rel=rel)
+            return self.conn.info.server_version < 120000
 
     def make_Float4(self, spec):
         return spec(self.make_float(spec, double=False))
@@ -480,9 +485,7 @@ class Faker:
     def make_JsonFloat(self, spec):
         # A float limited to what json accepts
         # this exponent should generate no inf
-        return float(
-            f"{choice('-+')}0.{randrange(1 << 20)}e{randrange(-15,15)}"
-        )
+        return float(f"{choice('-+')}0.{randrange(1 << 20)}e{randrange(-15,15)}")
 
     def schema_list(self, cls):
         while True:
@@ -494,6 +497,11 @@ class Faker:
                 # There may be rounding errors or problems with inf.
                 continue
 
+            # CRDB doesn't support arrays of json
+            # https://github.com/cockroachdb/cockroach/issues/23468
+            if self.conn.info.vendor == "CockroachDB" and scls in (Json, Jsonb):
+                continue
+
             schema = self.make_schema(scls)
             if schema is not None:
                 break
@@ -504,7 +512,16 @@ class Faker:
         # don't make empty lists because they regularly fail cast
         length = randrange(1, self.list_max_length)
         spec = spec[1]
-        return [self.make(spec) for i in range(length)]
+        while True:
+            rv = [self.make(spec) for i in range(length)]
+
+            # TODO multirange lists fail binary dump if the last element is
+            # empty and there is no type annotation. See xfail in
+            # test_multirange::test_dump_builtin_array
+            if rv and isinstance(rv[-1], Multirange) and not rv[-1]:
+                continue
+
+            return rv
 
     def example_list(self, spec):
         return [self.example(spec[1])]
@@ -544,7 +561,7 @@ class Faker:
 
             return l1 <= u2 and l2 <= u1
 
-        out = []
+        out: List[Range[Any]] = []
         for i in range(length):
             r = self.make_Range((Range, spec[1]), **kwargs)
             if r.isempty:
@@ -561,9 +578,7 @@ class Faker:
         return spec[0](sorted(out))
 
     def example_Multirange(self, spec):
-        return self.make_Multirange(
-            spec, length=1, empty_chance=0, no_bound_chance=0
-        )
+        return self.make_Multirange(spec, length=1, empty_chance=0, no_bound_chance=0)
 
     def make_Int4Multirange(self, spec):
         return self.make_Multirange((spec, Int4))
@@ -638,7 +653,7 @@ class Faker:
             return spec[0](empty=True)
 
         while True:
-            bounds = []
+            bounds: List[Union[Any, None]] = []
             while len(bounds) < 2:
                 if random() < no_bound_chance:
                     bounds.append(None)
@@ -699,6 +714,7 @@ class Faker:
             want = type(want)(want.lower, want.upper, want.bounds[0] + ")")
 
         # Normalise discrete ranges
+        unit: Union[dt.timedelta, int, None]
         if spec[1] is dt.date:
             unit = dt.timedelta(days=1)
         elif type(spec[1]) is type and issubclass(spec[1], int):
@@ -708,13 +724,20 @@ class Faker:
 
         if unit is not None:
             if want.lower is not None and not want.lower_inc:
-                want = type(want)(
-                    want.lower + unit, want.upper, "[" + want.bounds[1]
-                )
+                want = type(want)(want.lower + unit, want.upper, "[" + want.bounds[1])
             if want.upper_inc:
-                want = type(want)(
-                    want.lower, want.upper + unit, want.bounds[0] + ")"
-                )
+                want = type(want)(want.lower, want.upper + unit, want.bounds[0] + ")")
+
+        if spec[1] == (dt.datetime, True) and not want.isempty:
+            # work around https://bugs.python.org/issue45347
+            def fix_dt(x):
+                return x.astimezone(dt.timezone.utc) if x is not None else None
+
+            def fix_range(r):
+                return type(r)(fix_dt(r.lower), fix_dt(r.upper), r.bounds)
+
+            want = fix_range(want)
+            got = fix_range(got)
 
         assert got == want
 
@@ -740,7 +763,7 @@ class Faker:
         if not length:
             length = randrange(self.str_max_length)
 
-        rv = []
+        rv: List[int] = []
         while len(rv) < length:
             c = randrange(1, 128) if random() < 0.5 else randrange(1, 0x110000)
             if not (0xD800 <= c <= 0xDBFF or 0xDC00 <= c <= 0xDFFF):
@@ -760,14 +783,21 @@ class Faker:
         tz = self._make_tz(spec) if spec[1] else None
         return dt.time(h, m, s, ms, tz)
 
+    CRDB_TIMEDELTA_MAX = dt.timedelta(days=1281239)
+
     def make_timedelta(self, spec):
-        return choice([dt.timedelta.min, dt.timedelta.max]) * random()
+        if self.conn.info.vendor == "CockroachDB":
+            rng = [-self.CRDB_TIMEDELTA_MAX, self.CRDB_TIMEDELTA_MAX]
+        else:
+            rng = [dt.timedelta.min, dt.timedelta.max]
+
+        return choice(rng) * random()
 
     def schema_tuple(self, cls):
         # TODO: this is a complicated matter as it would involve creating
         # temporary composite types.
         # length = randrange(1, self.tuple_max_length)
-        # return (cls, self.choose_schema(ncols=length))
+        # return (cls, self.make_random_schema(ncols=length))
         return None
 
     def make_tuple(self, spec):
@@ -806,7 +836,7 @@ class Faker:
                 assert False, f"unknown rec type: {cls}"
 
         else:
-            cls = choice(scal_types)
+            cls = choice(scal_types)  # type: ignore[assignment]
             return self.make(cls)
 
     def _make_tz(self, spec):
@@ -819,7 +849,7 @@ class JsonFloat:
 
 
 def deep_import(name):
-    parts = deque(name.split("."))
+    parts = Deque(name.split("."))
     seen = []
     if not parts:
         raise ValueError("name must be a dot-separated name")
