@@ -11,7 +11,7 @@ from psycopg.pq import TransactionStatus
 from psycopg._compat import create_task
 from .test_pool_async import delay_connection, ensure_waiting
 
-pytestmark = [pytest.mark.asyncio]
+pytestmark = [pytest.mark.anyio]
 
 try:
     from psycopg_pool import AsyncNullConnectionPool  # noqa: F401
@@ -193,7 +193,6 @@ async def test_reset(dsn):
 
     async with AsyncNullConnectionPool(dsn, max_size=1, reset=reset) as p:
         async with p.connection() as conn:
-
             # Queue the worker so it will take the same connection a second time
             # instead of making a new one.
             t = create_task(worker())
@@ -226,7 +225,6 @@ async def test_reset_badstate(dsn, caplog):
 
     async with AsyncNullConnectionPool(dsn, max_size=1, reset=reset) as p:
         async with p.connection() as conn:
-
             t = create_task(worker())
             await ensure_waiting(p)
 
@@ -257,7 +255,6 @@ async def test_reset_broken(dsn, caplog):
 
     async with AsyncNullConnectionPool(dsn, max_size=1, reset=reset) as p:
         async with p.connection() as conn:
-
             t = create_task(worker())
             await ensure_waiting(p)
 
@@ -842,3 +839,62 @@ async def test_stats_connect(dsn, proxy, monkeypatch):
         assert stats.get("connections_errors", 0) == 0
         assert stats.get("connections_lost", 0) == 0
         assert 200 <= stats["connections_ms"] < 300
+
+
+@pytest.mark.skipif("sys.version_info < (3, 8)", reason="asyncio bug")
+async def test_cancellation_in_queue(dsn):
+    # https://github.com/psycopg/psycopg/issues/509
+
+    nconns = 3
+
+    async with AsyncNullConnectionPool(
+        dsn, min_size=0, max_size=nconns, timeout=1
+    ) as p:
+        await p.wait()
+
+        got_conns = []
+        ev = asyncio.Event()
+
+        async def worker(i):
+            try:
+                logging.info("worker %s started", i)
+                nonlocal got_conns
+
+                async with p.connection() as conn:
+                    logging.info("worker %s got conn", i)
+                    cur = await conn.execute("select 1")
+                    assert (await cur.fetchone()) == (1,)
+
+                    got_conns.append(conn)
+                    if len(got_conns) >= nconns:
+                        ev.set()
+
+                    await asyncio.sleep(5)
+
+            except BaseException as ex:
+                logging.info("worker %s stopped: %r", i, ex)
+                raise
+
+        # Start tasks taking up all the connections and getting in the queue
+        tasks = [asyncio.ensure_future(worker(i)) for i in range(nconns * 3)]
+
+        # wait until the pool has served all the connections and clients are queued.
+        await asyncio.wait_for(ev.wait(), 3.0)
+        for i in range(10):
+            if p.get_stats().get("requests_queued", 0):
+                break
+            else:
+                await asyncio.sleep(0.1)
+        else:
+            pytest.fail("no client got in the queue")
+
+        [task.cancel() for task in reversed(tasks)]
+        # Python 3.7 hangs on this statement, instead of timing out or returning
+        await asyncio.wait_for(asyncio.gather(*tasks, return_exceptions=True), 1.0)
+
+        stats = p.get_stats()
+        assert stats.get("requests_waiting", 0) == 0
+
+        async with p.connection() as conn:
+            cur = await conn.execute("select 1")
+            assert await cur.fetchone() == (1,)
