@@ -4,6 +4,7 @@ import pytest
 
 from psycopg import DataError, pq, sql
 from psycopg.adapt import PyFormat
+from psycopg._compat import ZoneInfo
 
 crdb_skip_datestyle = pytest.mark.crdb("skip", reason="set datestyle/intervalstyle")
 crdb_skip_negative_interval = pytest.mark.crdb("skip", reason="negative interval")
@@ -569,6 +570,28 @@ class TestTimeTz:
         cur.execute(f"select '{expr}'::timetz = %{fmt_in.value}", (as_time(val),))
         assert cur.fetchone()[0] is True
 
+    @pytest.mark.parametrize("fmt_in", PyFormat)
+    def test_dump_timetz_zoneinfo(self, conn, fmt_in):
+        t = dt.time(12, 0, tzinfo=ZoneInfo("Europe/Rome"))
+        with pytest.raises(DataError, match="Europe/Rome"):
+            conn.execute(f"select %{fmt_in.value}", (t,))
+
+    @pytest.mark.parametrize("fmt_in", PyFormat)
+    def test_dump_timetz_overflow(self, conn, fmt_in):
+        class MyTimeZone(dt.tzinfo):
+            def dst(self, dt_):
+                return None
+
+            def utcoffset(self, dt_):
+                return dt.timedelta(hours=25)
+
+            def tzname(self, dt_):
+                return "lol"
+
+        t = dt.time(12, 0, tzinfo=MyTimeZone())
+        with pytest.raises(ValueError):
+            conn.execute(f"select %{fmt_in.value}", (t,))
+
     @pytest.mark.parametrize(
         "val, expr, timezone",
         [
@@ -688,6 +711,8 @@ class TestInterval:
             ("-30d", "-1 month"),
             ("60d", "2 month"),
             ("-90d", "-3 month"),
+            ("186d", "6 mons 6 days"),
+            ("736d", "2 years 6 days"),
         ],
     )
     @pytest.mark.parametrize("fmt_out", pq.Format)
