@@ -13,10 +13,11 @@ from psycopg.adapt import PyFormat
 def test_uuid_dump(conn, fmt_in):
     val = "12345678123456781234567812345679"
     cur = conn.cursor()
-    cur.execute(f"select %{fmt_in} = %s::uuid", (UUID(val), val))
+    cur.execute(f"select %{fmt_in.value} = %s::uuid", (UUID(val), val))
     assert cur.fetchone()[0] is True
 
 
+@pytest.mark.crdb_skip("copy")
 @pytest.mark.parametrize("fmt_out", pq.Format)
 def test_uuid_load(conn, fmt_out):
     cur = conn.cursor(binary=fmt_out)
@@ -34,6 +35,7 @@ def test_uuid_load(conn, fmt_out):
     assert res == UUID(val)
 
 
+@pytest.mark.slow
 @pytest.mark.subprocess
 def test_lazy_load(dsn):
     script = f"""\
